@@ -18,13 +18,14 @@
 from __future__ import division, absolute_import, print_function
 
 import shlex
+import six
 
 from beets.plugins import BeetsPlugin
 from beets.ui import decargs, print_, Subcommand, UserError
 from beets.util import command_output, displayable_path, subprocess, \
-    bytestring_path
+    bytestring_path, MoveOperation, decode_commandline_path
 from beets.library import Item, Album
-import six
+
 
 PLUGIN = 'duplicates'
 
@@ -81,7 +82,7 @@ class DuplicatesPlugin(BeetsPlugin):
             help=u'report duplicates only if all attributes are set',
         )
         self._command.parser.add_option(
-            u'-k', u'--key',
+            u'-k', u'--key', dest='keys',
             action='append', metavar='KEY',
             help=u'report duplicates based on keys (use multiple times)',
         )
@@ -135,6 +136,11 @@ class DuplicatesPlugin(BeetsPlugin):
                     keys = ['mb_trackid', 'mb_albumid']
                 items = lib.items(decargs(args))
 
+            # If there's nothing to do, return early. The code below assumes
+            # `items` to be non-empty.
+            if not items:
+                return
+
             if path:
                 fmt = u'$path'
 
@@ -175,10 +181,10 @@ class DuplicatesPlugin(BeetsPlugin):
         """
         print_(format(item, fmt))
         if copy:
-            item.move(basedir=copy, copy=True)
+            item.move(basedir=copy, operation=MoveOperation.COPY)
             item.store()
         if move:
-            item.move(basedir=move, copy=False)
+            item.move(basedir=move)
             item.store()
         if delete:
             item.remove(delete=True)
@@ -197,7 +203,8 @@ class DuplicatesPlugin(BeetsPlugin):
         output as flexattr on a key that is the name of the program, and
         return the key, checksum tuple.
         """
-        args = [p.format(file=item.path) for p in shlex.split(prog)]
+        args = [p.format(file=decode_commandline_path(item.path))
+                for p in shlex.split(prog)]
         key = args[0]
         checksum = getattr(item, key, False)
         if not checksum:
@@ -205,7 +212,7 @@ class DuplicatesPlugin(BeetsPlugin):
                             u'computing checksum',
                             key, displayable_path(item.path))
             try:
-                checksum = command_output(args)
+                checksum = command_output(args).stdout
                 setattr(item, key, checksum)
                 item.store()
                 self._log.debug(u'computed checksum for {0} using {1}',
@@ -253,20 +260,19 @@ class DuplicatesPlugin(BeetsPlugin):
         "completeness" (objects with more non-null fields come first)
         and Albums are ordered by their track count.
         """
-        if tiebreak:
-            kind = 'items' if all(isinstance(o, Item)
-                                  for o in objs) else 'albums'
+        kind = 'items' if all(isinstance(o, Item) for o in objs) else 'albums'
+
+        if tiebreak and kind in tiebreak.keys():
             key = lambda x: tuple(getattr(x, k) for k in tiebreak[kind])
         else:
-            kind = Item if all(isinstance(o, Item) for o in objs) else Album
-            if kind is Item:
+            if kind == 'items':
                 def truthy(v):
                     # Avoid a Unicode warning by avoiding comparison
                     # between a bytes object and the empty Unicode
                     # string ''.
                     return v is not None and \
                         (v != '' if isinstance(v, six.text_type) else True)
-                fields = kind.all_keys()
+                fields = Item.all_keys()
                 key = lambda x: sum(1 for f in fields if truthy(getattr(x, f)))
             else:
                 key = lambda x: len(x.items())
@@ -312,7 +318,7 @@ class DuplicatesPlugin(BeetsPlugin):
                                     objs[0],
                                     displayable_path(o.path),
                                     displayable_path(missing.destination()))
-                    missing.move(copy=True)
+                    missing.move(operation=MoveOperation.COPY)
         return objs
 
     def _merge(self, objs):
