@@ -19,11 +19,10 @@
 from __future__ import division, absolute_import, print_function
 
 import os
-import re
 
 from beets.plugins import BeetsPlugin
 from beets import ui
-from beets import mediafile
+import mediafile
 from beets.library import Item
 from beets.util import displayable_path, normpath, syspath
 
@@ -42,15 +41,29 @@ def tag_data(lib, args):
             yield tag_data_emitter(item.path)
 
 
+def tag_fields():
+    fields = set(mediafile.MediaFile.readable_fields())
+    fields.add('art')
+    return fields
+
+
 def tag_data_emitter(path):
-    def emitter():
-        fields = list(mediafile.MediaFile.readable_fields())
-        fields.remove('images')
+    def emitter(included_keys):
+        if included_keys == '*':
+            fields = tag_fields()
+        else:
+            fields = included_keys
+        if 'images' in fields:
+            # We can't serialize the image data.
+            fields.remove('images')
         mf = mediafile.MediaFile(syspath(path))
         tags = {}
         for field in fields:
-            tags[field] = getattr(mf, field)
-        tags['art'] = mf.art is not None
+            if field == 'art':
+                tags[field] = mf.art is not None
+            else:
+                tags[field] = getattr(mf, field, None)
+
         # create a temporary Item to take advantage of __format__
         item = Item.from_path(syspath(path))
 
@@ -64,16 +77,15 @@ def library_data(lib, args):
 
 
 def library_data_emitter(item):
-    def emitter():
-        data = dict(item.formatted())
-        data.pop('path', None)  # path is fetched from item
+    def emitter(included_keys):
+        data = dict(item.formatted(included_keys=included_keys))
 
         return data, item
     return emitter
 
 
 def update_summary(summary, tags):
-    for key, value in tags.iteritems():
+    for key, value in tags.items():
         if key not in summary:
             summary[key] = value
         elif summary[key] != value:
@@ -96,7 +108,7 @@ def print_data(data, item=None, fmt=None):
 
     path = displayable_path(item.path) if item else None
     formatted = {}
-    for key, value in data.iteritems():
+    for key, value in data.items():
         if isinstance(value, list):
             formatted[key] = u'; '.join(value)
         if value is not None:
@@ -123,7 +135,7 @@ def print_data_keys(data, item=None):
     """
     path = displayable_path(item.path) if item else None
     formatted = []
-    for key, value in data.iteritems():
+    for key, value in data.items():
         formatted.append(key)
 
     if len(formatted) == 0:
@@ -184,18 +196,18 @@ class InfoPlugin(BeetsPlugin):
         included_keys = []
         for keys in opts.included_keys:
             included_keys.extend(keys.split(','))
-        key_filter = make_key_filter(included_keys)
+        # Drop path even if user provides it multiple times
+        included_keys = [k for k in included_keys if k != 'path']
 
         first = True
         summary = {}
         for data_emitter in data_collector(lib, ui.decargs(args)):
             try:
-                data, item = data_emitter()
+                data, item = data_emitter(included_keys or '*')
             except (mediafile.UnreadableFileError, IOError) as ex:
                 self._log.error(u'cannot read file: {0}', ex)
                 continue
 
-            data = key_filter(data)
             if opts.summarize:
                 update_summary(summary, data)
             else:
@@ -204,38 +216,9 @@ class InfoPlugin(BeetsPlugin):
                 if opts.keys_only:
                     print_data_keys(data, item)
                 else:
-                    print_data(data, item, opts.format)
+                    fmt = ui.decargs([opts.format])[0] if opts.format else None
+                    print_data(data, item, fmt)
                 first = False
 
         if opts.summarize:
             print_data(summary)
-
-
-def make_key_filter(include):
-    """Return a function that filters a dictionary.
-
-    The returned filter takes a dictionary and returns another
-    dictionary that only includes the key-value pairs where the key
-    glob-matches one of the keys in `include`.
-    """
-    if not include:
-        return identity
-
-    matchers = []
-    for key in include:
-        key = re.escape(key)
-        key = key.replace(r'\*', '.*')
-        matchers.append(re.compile(key + '$'))
-
-    def filter_(data):
-        filtered = dict()
-        for key, value in data.items():
-            if any(map(lambda m: m.match(key), matchers)):
-                filtered[key] = value
-        return filtered
-
-    return filter_
-
-
-def identity(val):
-    return val
