@@ -23,8 +23,8 @@ from beets.plugins import BeetsPlugin
 __author__ = 'baobab@heresiarch.info'
 __version__ = '1.1'
 
-PATTERN_THE = u'^[the]{3}\s'
-PATTERN_A = u'^[a][n]?\s'
+PATTERN_THE = u'^the\\s'
+PATTERN_A = u'^[a][n]?\\s'
 FORMAT = u'{0}, {1}'
 
 
@@ -54,14 +54,14 @@ class ThePlugin(BeetsPlugin):
                     self._log.error(u'invalid pattern: {0}', p)
                 else:
                     if not (p.startswith('^') or p.endswith('$')):
-                        self._log.warn(u'warning: \"{0}\" will not '
-                                       u'match string start/end', p)
+                        self._log.warning(u'warning: \"{0}\" will not '
+                                          u'match string start/end', p)
         if self.config['a']:
             self.patterns = [PATTERN_A] + self.patterns
         if self.config['the']:
             self.patterns = [PATTERN_THE] + self.patterns
         if not self.patterns:
-            self._log.warn(u'no patterns defined!')
+            self._log.warning(u'no patterns defined!')
 
     def unthe(self, text, pattern):
         """Moves pattern in the path format string or strips it
@@ -81,7 +81,7 @@ class ThePlugin(BeetsPlugin):
                 if self.config['strip']:
                     return r
                 else:
-                    fmt = self.config['format'].get(unicode)
+                    fmt = self.config['format'].as_str()
                     return fmt.format(r, t.strip()).strip()
         else:
             return u''
@@ -93,8 +93,8 @@ class ThePlugin(BeetsPlugin):
             for p in self.patterns:
                 r = self.unthe(text, p)
                 if r != text:
+                    self._log.debug(u'\"{0}\" -> \"{1}\"', text, r)
                     break
-            self._log.debug(u'\"{0}\" -> \"{1}\"', text, r)
             return r
         else:
             return u''
